@@ -59,14 +59,17 @@ est_predictiveness_cv <- function(fitted_values, y, full_y = NULL,
                                   na.rm = FALSE, ...) {
     # get the correct measure function; if not one of the supported ones, say so
     types <- c("accuracy", "auc", "deviance", "r_squared", "anova", "mse",
-               "cross_entropy")
+               "cross_entropy", "average_value", "sensitivity", "specificity",
+               "ppv", "npv")
     full_type <- types[pmatch(type, types)]
     if (is.na(full_type)) stop(
         paste0("We currently do not support the entered variable importance ",
                "parameter.")
     )
     measure_funcs <- c(measure_accuracy, measure_auc, measure_cross_entropy,
-                       measure_mse, NA, measure_mse, measure_cross_entropy)
+                       measure_mse, NA, measure_mse, measure_cross_entropy,
+                       measure_average_value, measure_sensitivity,
+                       measure_specificity, measure_ppv, measure_npv)
     measure_func <- measure_funcs[pmatch(type, types)]
     if (is.list(fitted_values)) {
         fitted_values_vector <- vector("numeric", length = length(y))
