@@ -17,7 +17,7 @@ measure_anova <- function(full, reduced, y, full_y = NULL,
                           ipc_weights = rep(1, length(y)),
                           ipc_fit_type = "external",
                           ipc_eif_preds = rep(1, length(y)),
-                          ipc_est_type = "aipw", scale = "identity",
+                          ipc_est_type = "aipw", scale = "logit",
                           na.rm = FALSE, nuisance_estimators = NULL,
                           a = NULL, ...) {
     if (is.null(full_y)) {
